type Cursor = {
    cursor?: object;
};
type Params = Cursor & {
    limit?: number;
    index?: string;
};
export interface DocumentsRetrieverResponse<T> {
    data: T[];
    metadata?: Cursor;
}
export declare enum ConditionTypeEnum {
    KEY = "KEY",
    KEY_BEGINS_WITH = "KEY_BEGINS_WITH",
    FILTER = "FILTER",
    CONTAINS = "CONTAINS",
    BEGINS_WITH = "BEGINS_WITH"
}
export declare class BaseCondition {
    type: ConditionTypeEnum;
    constructor(type: ConditionTypeEnum);
}
/**
 * KeyCondition.
 *
 * This represents a condition with `key == value` where key is an index
 *
 * @extends {BaseCondition}
 */
export declare class KeyCondition extends BaseCondition {
    criteria: {
        [keyName: string]: string | number | boolean;
    };
    constructor(criteria: {
        [keyName: string]: string | number | boolean;
    });
}
/**
 * KeyBeginsWithCondition.
 *
 * This represents a condition with `begins_with(sortKey, :sortKeyValue)` where keyName is a sort key
 *
 * @extends {BaseCondition}
 */
export declare class KeyBeginsWithCondition extends BaseCondition {
    criteria: {
        [keyName: string]: string | number | boolean;
    };
    constructor(criteria: {
        [keyName: string]: string | number | boolean;
    });
}
/**
 * filterCondition.
 *
 * This represents a condition with `key == value` on a non-key property
 *
 * @extends {BaseCondition}
 */
export declare class FilterCondition extends BaseCondition {
    criteria: {
        [propertyName: string]: string | number | boolean;
    };
    constructor(criteria: {
        [propertyName: string]: string | number | boolean;
    });
}
/**
 * containsCondition.
 *
 * This represents a condition where an array (key) contains a value
 *
 * @extends {BaseCondition}
 */
export declare class ContainsCondition extends BaseCondition {
    criteria: {
        [arrayName: string]: string | number | boolean;
    };
    constructor(criteria: {
        [arrayName: string]: string | number | boolean;
    });
}
export declare class BeginsWithCondition extends BaseCondition {
    criteria: {
        [arrayName: string]: string | number | boolean;
    };
    constructor(criteria: {
        [arrayName: string]: string | number | boolean;
    });
}
export interface ConditionalItems<T> {
    item: T;
    conditions: BaseCondition[];
}
export declare abstract class IGenericRepository<T extends object> {
    abstract query(conditions: BaseCondition[], params: Params): Promise<DocumentsRetrieverResponse<T>>;
    abstract get(id: unknown): Promise<T>;
    abstract bulkGet(items: T[]): Promise<(T | undefined)[]>;
    abstract create(item: T): Promise<T>;
    abstract bulkCreate(items: T[]): Promise<T[]>;
    abstract update(id: unknown, item: T, conditions?: BaseCondition[]): Promise<T>;
    abstract delete(id: unknown, conditions?: BaseCondition[]): Promise<void>;
    abstract bulkDelete(conditionalItems: ConditionalItems<T>[]): Promise<void>;
}
export {};
