"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IGenericRepository = exports.BeginsWithCondition = exports.ContainsCondition = exports.FilterCondition = exports.KeyBeginsWithCondition = exports.KeyCondition = exports.BaseCondition = exports.ConditionTypeEnum = void 0;
var ConditionTypeEnum;
(function (ConditionTypeEnum) {
    ConditionTypeEnum["KEY"] = "KEY";
    ConditionTypeEnum["KEY_BEGINS_WITH"] = "KEY_BEGINS_WITH";
    ConditionTypeEnum["FILTER"] = "FILTER";
    ConditionTypeEnum["CONTAINS"] = "CONTAINS";
    ConditionTypeEnum["BEGINS_WITH"] = "BEGINS_WITH";
})(ConditionTypeEnum = exports.ConditionTypeEnum || (exports.ConditionTypeEnum = {}));
class BaseCondition {
    constructor(type) {
        this.type = type;
    }
}
exports.BaseCondition = BaseCondition;
/**
 * KeyCondition.
 *
 * This represents a condition with `key == value` where key is an index
 *
 * @extends {BaseCondition}
 */
class KeyCondition extends BaseCondition {
    constructor(criteria) {
        super(ConditionTypeEnum.KEY);
        this.criteria = criteria;
    }
}
exports.KeyCondition = KeyCondition;
/**
 * KeyBeginsWithCondition.
 *
 * This represents a condition with `begins_with(sortKey, :sortKeyValue)` where keyName is a sort key
 *
 * @extends {BaseCondition}
 */
class KeyBeginsWithCondition extends BaseCondition {
    constructor(criteria) {
        super(ConditionTypeEnum.KEY_BEGINS_WITH);
        this.criteria = criteria;
    }
}
exports.KeyBeginsWithCondition = KeyBeginsWithCondition;
/**
 * filterCondition.
 *
 * This represents a condition with `key == value` on a non-key property
 *
 * @extends {BaseCondition}
 */
class FilterCondition extends BaseCondition {
    constructor(criteria) {
        super(ConditionTypeEnum.FILTER);
        this.criteria = criteria;
    }
}
exports.FilterCondition = FilterCondition;
/**
 * containsCondition.
 *
 * This represents a condition where an array (key) contains a value
 *
 * @extends {BaseCondition}
 */
class ContainsCondition extends BaseCondition {
    constructor(criteria) {
        super(ConditionTypeEnum.CONTAINS);
        this.criteria = criteria;
    }
}
exports.ContainsCondition = ContainsCondition;
class BeginsWithCondition extends BaseCondition {
    // Will be use for the filterExpression
    constructor(criteria) {
        super(ConditionTypeEnum.BEGINS_WITH);
        this.criteria = criteria;
    }
}
exports.BeginsWithCondition = BeginsWithCondition;
class IGenericRepository {
}
exports.IGenericRepository = IGenericRepository;
