import { IGenericRepository, BaseCondition, ConditionalItems } from '../../abstracts/generic-repository.abstract';
import { AttributeValue } from '@aws-sdk/client-dynamodb';
export interface IFormatedConditions {
    keyConditionExpression: string;
    filterExpression: string;
    conditionExpression: string;
    expressionAttributeValues: {
        [key: string]: AttributeValue;
    };
    expressionAttributeNames: {
        [key: string]: string;
    };
}
export interface IDynamodbGenericRepositoryConfiguration {
    dataRetentionInDays: number | undefined;
    awsRegion: string;
    maxRetries: number;
    httpTimeoutMs: number;
    httpConnectTimeoutMs: number;
    isDdbLocal: boolean;
    localDdbHost: string;
}
/**
 * IDynamodbGenericRepositoryParameters.
 */
export interface IDynamodbGenericRepositoryParameters {
    /**
     * @type {string}
     * @description name of the DDB table
     */
    tableName: string;
    /**
     * @type {boolean}
     * @description The scan operation must be used for query if there is no range key in the table (such as a root resource)
     */
    useScanForQuery: boolean;
    /**
     * @type {IDynamodbGenericRepositoryConfiguration}
     * @description The DDB configuration
     */
    config: IDynamodbGenericRepositoryConfiguration;
}
export declare class DynamodbGenericRepository<T extends object, TKey> implements IGenericRepository<T> {
    private params;
    private dataRetentionInDays;
    private ddbClient;
    private ddbDocClient;
    private ttlAttributeName;
    private hideItemsWithTTLAttribute;
    private isInit;
    private keySchema;
    constructor(params: IDynamodbGenericRepositoryParameters);
    /**
     * init.
     *
     * This is need since the constructor cannot be async.
     * This function is private and called in each entrypoint (public method) for backward compatibility.
     */
    private _init;
    _formatConditions(conditions: BaseCondition[]): IFormatedConditions;
    /**
     *
     * @return {number} Epoch time when  to delete the item based on the item retention delay
     */
    private _getTTLAttributeValue;
    get(id: TKey): Promise<T | undefined>;
    bulkGet(items: T[]): Promise<(T | undefined)[]>;
    query(conditions: BaseCondition[], params: {
        limit: number;
        cursor?: object;
        index?: string;
    }): Promise<{
        data: T[];
        metadata: {
            cursor: object;
        };
    }>;
    create(item: T): Promise<T>;
    bulkCreate(items: T[]): Promise<T[]>;
    update(id: TKey, item: T, conditions?: BaseCondition[]): Promise<T>;
    delete(id: TKey, conditions?: BaseCondition[]): Promise<void>;
    bulkDelete(conditionalItems: ConditionalItems<T>[]): Promise<void>;
}
