"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamodbGenericRepository = void 0;
const _ = __importStar(require("lodash"));
const generic_repository_abstract_1 = require("../../abstracts/generic-repository.abstract");
const client_dynamodb_1 = require("@aws-sdk/client-dynamodb");
const lib_dynamodb_1 = require("@aws-sdk/lib-dynamodb");
const class_transformer_1 = require("class-transformer");
// see https://docs.aws.amazon.com/sdk-for-javascript/v3/developer-guide/javascript_dynamodb_code_examples.html
class DynamodbGenericRepository {
    constructor(params) {
        this.params = params;
        this.dataRetentionInDays = this.params.config.dataRetentionInDays;
        this.hideItemsWithTTLAttribute = !!this.dataRetentionInDays;
        this.isInit = false;
        const options = {
            region: this.params.config.awsRegion,
            retryDelayOptions: {
                base: 50, // Default is 50 for DynamoDB and not 100, see https://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/Config.html#retryDelayOptions-property
            },
            maxRetries: this.params.config.maxRetries,
            httpOptions: {
                timeout: this.params.config.httpTimeoutMs,
                connectTimeout: this.params.config.httpConnectTimeoutMs,
            },
        };
        if (this.params.config.isDdbLocal) {
            options.endpoint = `http://${this.params.config.localDdbHost}:8000`;
            options.region = 'FakeAwsRegionForLocalDDB';
            options.credentials = {
                accessKeyId: 'FakeAwsAccessKeyLocalDDB',
                secretAccessKey: 'FakeAwsSecretAccessKeyLocalDDB',
            };
        }
        this.ddbClient = new client_dynamodb_1.DynamoDB(options);
        const marshallOptions = {
            // Whether to automatically convert empty strings, blobs, and sets to `null`.
            convertEmptyValues: false,
            // Whether to remove undefined values while marshalling.
            removeUndefinedValues: true,
            // Whether to convert typeof object to map attribute.
            convertClassInstanceToMap: true, // false, by default.
        };
        const unmarshallOptions = {
            // Whether to return numbers as a string instead of converting them to native JavaScript numbers.
            wrapNumbers: false, // false, by default.
        };
        // Create the DynamoDB document client.
        this.ddbDocClient = lib_dynamodb_1.DynamoDBDocumentClient.from(this.ddbClient, {
            marshallOptions,
            unmarshallOptions,
        });
    }
    /**
     * init.
     *
     * This is need since the constructor cannot be async.
     * This function is private and called in each entrypoint (public method) for backward compatibility.
     */
    _init() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isInit) {
                const describeTableAndTTLInput = {
                    TableName: this.params.tableName,
                };
                const describeTTLOutput = yield this.ddbClient.send(new client_dynamodb_1.DescribeTimeToLiveCommand(describeTableAndTTLInput));
                const describeTableOutput = yield this.ddbClient.send(new client_dynamodb_1.DescribeTableCommand(describeTableAndTTLInput));
                if (this.dataRetentionInDays && ((_a = describeTTLOutput === null || describeTTLOutput === void 0 ? void 0 : describeTTLOutput.TimeToLiveDescription) === null || _a === void 0 ? void 0 : _a.TimeToLiveStatus) !== 'ENABLED') {
                    throw new Error(`TimeToLiveSpecification is not ENABLED on the ${this.params.tableName} table. dataRetentionInDays cannot be used`);
                }
                this.ttlAttributeName = (_b = describeTTLOutput === null || describeTTLOutput === void 0 ? void 0 : describeTTLOutput.TimeToLiveDescription) === null || _b === void 0 ? void 0 : _b.AttributeName;
                this.keySchema = describeTableOutput.Table.KeySchema;
                this.isInit = true;
            }
        });
    }
    _formatConditions(conditions) {
        const keyConditionExpression = [];
        const filterExpression = this.hideItemsWithTTLAttribute
            ? [`attribute_not_exists(${this.ttlAttributeName})`]
            : [];
        const expressionAttributeValues = {};
        const expressionAttributeNames = {};
        conditions
            .filter((condition) => condition.type == generic_repository_abstract_1.ConditionTypeEnum.KEY)
            .forEach((condition) => {
            Object.entries(condition.criteria).forEach(([key, value]) => {
                expressionAttributeNames[`#${key.replace(/\./, '_')}`] = key;
                keyConditionExpression.push(`#${key.replace(/\./, '_')} = :${key.replace(/\./, '_')}`);
                expressionAttributeValues[`:${key.replace(/\./, '_')}`] = value;
            });
        });
        conditions
            .filter((condition) => condition.type == generic_repository_abstract_1.ConditionTypeEnum.KEY_BEGINS_WITH)
            .forEach((condition) => {
            Object.entries(condition.criteria).forEach(([key, value]) => {
                expressionAttributeNames[`#${key.replace(/\./, '_')}`] = key;
                keyConditionExpression.push(`begins_with(#${key.replace(/\./, '_')}, :${key.replace(/\./, '_')})`);
                expressionAttributeValues[`:${key.replace(/\./, '_')}`] = value;
            });
        });
        conditions
            .filter((condition) => condition.type == generic_repository_abstract_1.ConditionTypeEnum.FILTER)
            .forEach((condition) => {
            Object.entries(condition.criteria).forEach(([key, value]) => {
                key.split('.').forEach((k) => (expressionAttributeNames[`#${k}`] = k));
                const namedKey = key.replace(/([^.]+)./g, '#$&');
                filterExpression.push(`${namedKey} = :${key.replace(/\./, '_')}`);
                expressionAttributeValues[`:${key.replace(/\./, '_')}`] = value;
            });
        });
        conditions
            .filter((condition) => condition.type == generic_repository_abstract_1.ConditionTypeEnum.CONTAINS)
            .forEach((condition) => {
            Object.entries(condition.criteria).forEach(([key, value]) => {
                key.split('.').forEach((k) => (expressionAttributeNames[`#${k}`] = k));
                const namedKey = key.replace(/([^.]+)./g, '#$&');
                filterExpression.push(`contains(${namedKey}, :${key.replace(/\./, '_')})`);
                expressionAttributeValues[`:${key.replace(/\./, '_')}`] = value;
            });
        });
        conditions
            .filter((condition) => condition.type == generic_repository_abstract_1.ConditionTypeEnum.BEGINS_WITH)
            .forEach((condition) => {
            Object.entries(condition.criteria).forEach(([key, value]) => {
                key.split('.').forEach((k) => (expressionAttributeNames[`#${k}`] = k));
                const namedKey = key.replace(/([^.]+)./g, '#$&');
                filterExpression.push(`begins_with(${namedKey}, :${key.replace(/\./, '_')})`);
                expressionAttributeValues[`:${key.replace(/\./, '_')}`] = value;
            });
        });
        return {
            keyConditionExpression: _.isEmpty(keyConditionExpression)
                ? undefined
                : keyConditionExpression.join(' AND '),
            filterExpression: _.isEmpty(filterExpression) ? undefined : filterExpression.join(' AND '),
            conditionExpression: _.isEmpty(keyConditionExpression) && _.isEmpty(filterExpression)
                ? undefined
                : keyConditionExpression.concat(filterExpression).join(' AND '),
            expressionAttributeValues: _.isEmpty(expressionAttributeValues) ? undefined : expressionAttributeValues,
            expressionAttributeNames: _.isEmpty(expressionAttributeNames) ? undefined : expressionAttributeNames,
        };
    }
    /**
     *
     * @return {number} Epoch time when  to delete the item based on the item retention delay
     */
    _getTTLAttributeValue() {
        return Math.floor(Date.now() / 1000) + this.dataRetentionInDays * 24 * 60 * 60;
    }
    get(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            const getParams = {
                TableName: this.params.tableName,
                Key: id,
            };
            const getResult = yield this.ddbDocClient.send(new lib_dynamodb_1.GetCommand(getParams));
            const item = _.get(getResult, 'Item');
            if (this.hideItemsWithTTLAttribute && _.get(item, this.ttlAttributeName)) {
                return undefined;
            }
            return item;
        });
    }
    bulkGet(items) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            if (items.length > 100) {
                throw new Error('TooManyItemsException');
            }
            const itemsToGet = [];
            const data = [];
            items.forEach((item) => {
                const itemKey = {};
                this.keySchema.forEach((key) => {
                    itemKey[key.AttributeName] = item[key.AttributeName];
                });
                itemsToGet.push({
                    Get: {
                        Key: itemKey,
                        TableName: this.params.tableName,
                    },
                });
            });
            const transactParams = {
                TransactItems: itemsToGet,
            };
            const result = yield this.ddbDocClient.send(new lib_dynamodb_1.TransactGetCommand(transactParams));
            result.Responses.forEach((response) => {
                var _a;
                let item;
                if (this.hideItemsWithTTLAttribute && ((_a = response === null || response === void 0 ? void 0 : response.Item) === null || _a === void 0 ? void 0 : _a[this.ttlAttributeName])) {
                    item = undefined;
                }
                else {
                    item = response.Item;
                }
                data.push(item);
            });
            return data;
        });
    }
    query(conditions, params) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            const { limit, cursor, index } = params;
            const formatedConditions = this._formatConditions(conditions);
            const Command = this.params.useScanForQuery ? lib_dynamodb_1.ScanCommand : lib_dynamodb_1.QueryCommand;
            const data = [];
            let lastEvaluatedKey = cursor;
            const queryParams = {
                TableName: this.params.tableName,
                IndexName: index,
                KeyConditionExpression: formatedConditions.keyConditionExpression,
                ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                ExclusiveStartKey: lastEvaluatedKey,
                FilterExpression: formatedConditions.filterExpression,
                Limit: limit - data.length,
            };
            do {
                const result = yield this.ddbDocClient.send(new Command(Object.assign(Object.assign({}, queryParams), { ExclusiveStartKey: lastEvaluatedKey, Limit: limit - data.length })));
                lastEvaluatedKey = result.LastEvaluatedKey;
                result.Items.forEach((item) => {
                    if (data.length < limit) {
                        data.push(item);
                    }
                });
            } while (data.length < limit && lastEvaluatedKey);
            return {
                data: data,
                metadata: {
                    cursor: lastEvaluatedKey,
                },
            };
        });
    }
    create(item) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            const createdItem = _.cloneDeep(item);
            const putParams = {
                TableName: this.params.tableName,
                Item: createdItem,
            };
            yield this.ddbDocClient.send(new lib_dynamodb_1.PutCommand(putParams));
            return createdItem;
        });
    }
    bulkCreate(items) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            if (items.length > 100) {
                throw new Error('TooManyItemsException');
            }
            const createdItems = _.cloneDeep(items);
            const putItems = [];
            createdItems.forEach((item) => {
                putItems.push({
                    Put: {
                        Item: item,
                        TableName: this.params.tableName,
                    },
                });
            });
            const transactPutParams = {
                TransactItems: putItems,
            };
            yield this.ddbDocClient.send(new lib_dynamodb_1.TransactWriteCommand(transactPutParams));
            return createdItems;
        });
    }
    update(id, item, conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            const itemToUpdate = yield this.get(id);
            if (!itemToUpdate) {
                throw new Error('NotFoundException');
            }
            conditions = conditions || [];
            conditions.push(new generic_repository_abstract_1.KeyCondition((0, class_transformer_1.instanceToPlain)(id)));
            const formatedConditions = this._formatConditions(conditions);
            const updatedItem = _.cloneDeep(item);
            const putParams = {
                TableName: this.params.tableName,
                ConditionExpression: formatedConditions.conditionExpression,
                ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                Item: updatedItem,
            };
            yield this.ddbDocClient.send(new lib_dynamodb_1.PutCommand(putParams));
            return updatedItem;
        });
    }
    delete(id, conditions) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            const itemToDelete = yield this.get(id);
            if (!itemToDelete) {
                throw new Error('NotFoundException');
            }
            if (this.dataRetentionInDays) {
                itemToDelete[this.ttlAttributeName] = this._getTTLAttributeValue();
                conditions = conditions || [];
                conditions.push(new generic_repository_abstract_1.KeyCondition((0, class_transformer_1.instanceToPlain)(id)));
                const formatedConditions = this._formatConditions(conditions);
                const putParams = {
                    TableName: this.params.tableName,
                    ConditionExpression: formatedConditions.conditionExpression,
                    ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                    ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                    Item: itemToDelete,
                };
                yield this.ddbDocClient.send(new lib_dynamodb_1.PutCommand(putParams));
            }
            else {
                conditions = conditions || [];
                const formatedConditions = this._formatConditions(conditions);
                const deleteParams = {
                    TableName: this.params.tableName,
                    ConditionExpression: formatedConditions.conditionExpression,
                    ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                    ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                    Key: id,
                };
                yield this.ddbDocClient.send(new lib_dynamodb_1.DeleteCommand(deleteParams));
            }
            return;
        });
    }
    bulkDelete(conditionalItems) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._init();
            if (conditionalItems.length > 100) {
                throw new Error('TooManyItemsException');
            }
            const itemsToDelete = [];
            conditionalItems.forEach(({ item, conditions }) => {
                if (this.dataRetentionInDays) {
                    const itemKey = {};
                    this.keySchema.forEach((key) => {
                        itemKey[key.AttributeName] = item[key.AttributeName];
                    });
                    conditions = conditions || [];
                    conditions.push(new generic_repository_abstract_1.KeyCondition((0, class_transformer_1.instanceToPlain)(itemKey)));
                    const formatedConditions = this._formatConditions(conditions);
                    item[this.ttlAttributeName] = this._getTTLAttributeValue();
                    itemsToDelete.push({
                        Put: {
                            Item: item,
                            ConditionExpression: formatedConditions.conditionExpression,
                            ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                            ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                            TableName: this.params.tableName,
                        },
                    });
                }
                else {
                    const itemKey = {};
                    this.keySchema.forEach((key) => {
                        itemKey[key.AttributeName] = item[key.AttributeName];
                    });
                    const formatedConditions = this._formatConditions(conditions);
                    itemsToDelete.push({
                        Delete: {
                            Key: itemKey,
                            ConditionExpression: formatedConditions.conditionExpression,
                            ExpressionAttributeValues: formatedConditions.expressionAttributeValues,
                            ExpressionAttributeNames: formatedConditions.expressionAttributeNames,
                            TableName: this.params.tableName,
                        },
                    });
                }
            });
            const transactParams = {
                TransactItems: itemsToDelete,
            };
            yield this.ddbDocClient.send(new lib_dynamodb_1.TransactWriteCommand(transactParams));
            return;
        });
    }
}
exports.DynamodbGenericRepository = DynamodbGenericRepository;
